%*****************************************************************************80
%
%% Evaluates the objective function for problem 18.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Chebyquad(what,x)
  global Nf_eval Ng_eval Nh_eval Chebyquad_n ;

  if isempty(Chebyquad_n)
    Chebyquad_n = 2;
  end

  switch what

  case 'setup'
    if x < 1 
      error(['Chebyquad(''setup'', ' x ') argument must be >= 1']) ; 
    end
    Chebyquad_n = x ;

  case {'name','title'}
    RES = 'The Chebyquad Function.' ;

  case {'n','size'}
    RES = Chebyquad_n ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = zeros ( Chebyquad_n, 1 );
    for i = 1 : Chebyquad_n
      RES(i) = i / ( Chebyquad_n + 1 );
    end

  case 'solution'
    RES = [] ;
    switch Chebyquad_n
      case 2; RES = [ 0.2113249, 0.7886751 ]';
      case 4; RES = [ 0.1026728, 0.4062037, 0.5937963, 0.8973272 ]';
      case 6; RES = [ 0.066877, 0.288741, 0.366682, 0.633318, 0.711259, 0.933123 ]';
      case 8; RES = [ 0.043153, 0.193091, 0.266329, 0.500000, 0.500000, 0.733671, 0.806910, 0.956847 ]';
    end

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Chebyquad Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function fvec = eval_fvec ( x )
  global Chebyquad_n ;
  n    = Chebyquad_n ;

  fvec = zeros(1,n) ;

  for j=1:n
    t1 = 1 ;
    t2 = 2 * x(j) - 1 ;
    t  = 2 * t2 ;
    for i=1:n
      fvec(i) = fvec(i) + t2 ;
      th = t * t2 - t1 ;
      t1 = t2 ;
      t2 = th ;
    end
  end

  for i=1:n
    fvec(i) = fvec(i) / n ;
    if mod( i, 2 ) == 0
      fvec(i) = fvec(i) + 1 / ( i^2 - 1 ) ;
    end
  end
end

%*****************************************************************************80
function f = eval_f ( x )
  global Chebyquad_n ;
  n    = Chebyquad_n ;
  fvec = eval_fvec( x ) ;
  f    = dot( fvec, fvec ) ;
end

%*****************************************************************************80
function g = eval_g ( x )
  global Chebyquad_n ;
  n = Chebyquad_n ;
  g = zeros ( 1, n );
%
%  Compute FVEC.
%
  fvec = eval_fvec( x ) ;
%
%  Compute G.
%
  for j = 1 : n
    g(j) = 0.0;
    t1   = 1.0;
    t2   = 2.0 * x(j) - 1.0;
    t    = 2.0 * t2;
    s1   = 0.0;
    s2   = 2.0;
    for i = 1 : n
      g(j) = g(j) + fvec(i) * s2;
      th = 4.0 * t2 + t * s2 - s1;
      s1 = s2;
      s2 = th;
      th = t * t2 - t1;
      t1 = t2;
      t2 = th;
    end
  end
  g = (2.0/n) * g ;
end

%*****************************************************************************80
function h = eval_h ( x )
  global Chebyquad_n ;
  n = Chebyquad_n ;
  h = zeros ( n, n );
  fvec = eval_fvec( x ) ;

  d1 = 1.0 / n;
  d2 = 2.0 * d1;

  for j = 1 : n

    h(j,j)  = 4.0 * d1;
    t1      = 1.0;
    t2      = 2.0 * x(j) - 1.0;
    t       = 2.0 * t2;
    s1      = 0.0;
    s2      = 2.0;
    p1      = 0.0;
    p2      = 0.0;
    gvec(1) = s2;

    for i = 2 : n
      th = 4.0 * t2 + t * s2 - s1;
      s1 = s2;
      s2 = th;
      th = t * t2 - t1;
      t1 = t2;
      t2 = th;
      th = 8.0 * s1 + t * p2 - p1;
      p1 = p2;
      p2 = th;
      gvec(i) = s2;
      h(j,j)  = h(j,j) + fvec(i) * th + d1 * s2 * s2;
    end

    h(j,j) = d2 * h(j,j);

    for k = 1 : j - 1

      h(j,k) = 0.0;
      tt1    = 1.0;
      tt2    = 2.0 * x(k) - 1.0;
      tt     = 2.0 * tt2;
      ss1    = 0.0;
      ss2    = 2.0;

      for i = 1 : n
        h(j,k) = h(j,k) + ss2 * gvec(i);
        tth = 4.0 * tt2 + tt * ss2 - ss1;
        ss1 = ss2;
        ss2 = tth;
        tth = tt * tt2 - tt1;
        tt1 = tt2;
        tt2 = tth;
      end

      h(j,k) = d2 * d1 * h(j,k);
    end
  end

  for i = 1 : n
    h(i,i+1:n) = h(i+1:n,i);
  end
end
