%*****************************************************************************80
%
%% Evaluates the objective function for problem 17.
%
%  Licensing:
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%    24 December 2011
%
%  Author:
%    Enrico Bertolazzi
%    Dipartimento di Ingegneria Meccanica e Strutturale
%    Universita` degli Studi di Trento
%    Via Mesiano 77, I-38123 Trento, Italy
%    email: enrico.bertolazzi@unitn.it
%
%    based on a code of John Burkardt (http://people.sc.fsu.edu/~jburkardt/)
%
%  Reference:
%    Richard Brent,
%    Algorithms for Minimization with Derivatives,
%    Dover, 2002,
%    ISBN: 0-486-41998-3,
%    LC: QA402.5.B74.
%
function RES = Wood(what,x)
  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The Wood Function.' ;

  case {'n','size'}
    RES = 4 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ -3.0, -1.0, -3.0, -1.0 ]' ;

  case 'solution'
    RES = [ 1.0, 1.0, 1.0, 1.0 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h(x ) ;

  otherwise
    error(['The Wood Function, bad argument:' what]) ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  f1 = x(2) - x(1) * x(1);
  f2 = 1.0 - x(1);
  f3 = x(4) - x(3) * x(3);
  f4 = 1.0 - x(3);
  f5 = x(2) + x(4) - 2.0;
  f6 = x(2) - x(4);

  f = 100.0 * f1 * f1 ...
    +         f2 * f2 ...
    +  90.0 * f3 * f3 ...
    +         f4 * f4 ...
    +  10.0 * f5 * f5 ...
    +   0.1 * f6 * f6 ;
end

%*****************************************************************************80
function g = eval_g ( x )
  g = zeros ( 1, 4 );

  f1 = x(2) - x(1) * x(1);
  f2 = 1.0 - x(1);
  f3 = x(4) - x(3)^2;
  f4 = 1.0 - x(3);
  f5 = x(2) + x(4) - 2.0;
  f6 = x(2) - x(4);

  df1dx1 = - 2.0 * x(1);
  df1dx2 =   1.0;
  df2dx1 = - 1.0;
  df3dx3 = - 2.0 * x(3);
  df3dx4 =   1.0;
  df4dx3 = - 1.0;
  df5dx2 =   1.0;
  df5dx4 =   1.0;
  df6dx2 =   1.0;
  df6dx4 = - 1.0;

  g(1) = 2.0 * ( 100.0 * f1 * df1dx1 + f2 * df2dx1 );
  g(2) = 2.0 * ( 100.0 * f1 * df1dx2 + 10.0 * f5 * df5dx2 + 0.1 * f6 * df6dx2 );
  g(3) = 2.0 * ( 90.0 * f3 * df3dx3 + f4 * df4dx3 );
  g(4) = 2.0 * ( 90.0 * f3 * df3dx4 + 10.0 * f5 * df5dx4 + 0.1 * f6 * df6dx4 );

end

%*****************************************************************************80
function h = eval_h ( x )
  h = zeros ( 4, 4 );

  f1 = x(2) - x(1)^2;
  f2 = 1.0 - x(1);
  f3 = x(4) - x(3)^2;
  f4 = 1.0 - x(3);
  f5 = x(2) + x(4) - 2.0;
  f6 = x(2) - x(4);

  df1dx1 = - 2.0 * x(1);
  df1dx2 =   1.0;
  df2dx1 = - 1.0;
  df3dx3 = - 2.0 * x(3);
  df3dx4 =   1.0;
  df4dx3 = - 1.0;
  df5dx2 =   1.0;
  df5dx4 =   1.0;
  df6dx2 =   1.0;
  df6dx4 = - 1.0;

  d2f1dx11 = - 2.0;
  d2f3dx33 = - 2.0;

  h(1,1) = 200.0 * df1dx1 * df1dx1 + 200.0 * f1 * d2f1dx11 + 2.0 * df2dx1 * df2dx1;
  h(1,2) = 200.0 * df1dx2 * df1dx1;
  h(1,3) = 0.0;
  h(1,4) = 0.0;

  h(2,1) = 200.0 * df1dx1 * df1dx2;
  h(2,2) = 200.0 * df1dx2 * df1dx2 + 20.0 * df5dx2 * df5dx2 + 0.2 * df6dx2 * df6dx2;
  h(2,3) = 0.0;
  h(2,4) = 20.0 * df5dx4 * df5dx2 + 0.2 * df6dx4 * df6dx2;

  h(3,1) = 0.0;
  h(3,2) = 0.0;
  h(3,3) = 180.0 * df3dx3 * df3dx3 + 180.0 * f3 * d2f3dx33 + 2.0 * df4dx3 * df4dx3;
  h(3,4) = 180.0 * df3dx4 * df3dx3;

  h(4,1) = 0.0;
  h(4,2) = 20.0 * df5dx2 * df5dx4 + 0.2 * df6dx2 * df6dx4;
  h(4,3) = 180.0 * df3dx3 * df3dx4;
  h(4,4) = 180.0 * df3dx4 * df3dx4 + 20.0 * df5dx4 * df5dx4 + 0.2 * df6dx4 * df6dx4;
end
