%*****************************************************************************80
%
%% MCKINNON computes the McKinnon function.
%
%  Discussion:
%
%    This function has a global minimizer:
%
%      X* = ( 0.0, -1.0 ), F(X*) = 0.
%
%    There are three parameters, TAU, THETA and PHI.
%
%    1 < TAU, then F is strictly convex.
%             and F has continuous first derivatives.
%    2 < TAU, then F has continuous second derivatives.
%    3 < TAU, then F has continuous third derivatives.
%
%    However, this function can cause the Nelder-Mead optimization
%    algorithm to "converge" to a point which is not the minimizer
%    of the function F.
%
%    Sample parameter values which cause problems for Nelder-Mead 
%    include:
%
%      TAU = 1, THETA = 15, PHI =  10;
%      TAU = 2, THETA =  6, PHI =  60;
%      TAU = 3, THETA =  6, PHI = 400;
%
%    To get the bad behavior, we also assume the initial simplex has the form
%
%      X1 = (0,0),
%      X2 = (1,1),
%      X3 = (A,B), 
%
%    where 
%
%      A = (1+sqrt(33))/8 =  0.84307...
%      B = (1-sqrt(33))/8 = -0.59307...
%
%  Licensing:
%
%    This code is distributed under the GNU LGPL license.
%
%  Modified:
%
%    09 February 2008
%
%  Author:
%
%    John Burkardt
%
%  Reference:
%
%    Ken McKinnon,
%    Convergence of the Nelder-Mead simplex method to a nonstationary point,
%    SIAM Journal on Optimization,
%    Volume 9, Number 1, 1998, pages 148-158.
%
%  Parameters:
%
%    Input, real X(2), the argument of the function.
%
%    Output, real F, the value of the function at X.
%
function RES = mckinnon( what, x )

  global Nf_eval Ng_eval Nh_eval ;
  switch what

  case {'name','title'}
    RES = 'The mckinnon function.' ;

  case {'n','size'}
    RES = 2 ;

  case 'm'
    RES = 1 ;

  case {'init','start'}
    RES = [ 1, 1 ]' ;

  case 'solution'
    RES = [ 0, -1 ]' ;

  case 'f'
    Nf_eval = Nf_eval+1 ;
    RES = eval_f( x ) ;

  case 'Df'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x ) ;

  case 'F'
    Ng_eval = Ng_eval+1 ;
    RES = eval_g( x )' ;

  case {'JF', 'DDf'}
    Nh_eval = Nh_eval+1;
    RES = eval_h( x ) ;

  otherwise
    error 'The mckinnon function, bad argument: %1', what ;
  end ;

end

%*****************************************************************************80
function f = eval_f ( x )
  tau   = 2.0;
  theta = 6.0;
  phi   = 60.0;

  if ( x(1) <= 0.0 )
    t2 = (-x(1)) ^ tau;
    f  = t2 * theta * phi + x(2) * (1 + x(2));
  else
    t1 = x(1) ^ tau;
    f  = theta * t1 + x(2) * (1 + x(2));
  end
end

%*****************************************************************************80
function g = eval_g ( x )
  tau   = 2.0;
  theta = 6.0;
  phi   = 60.0;

  g  = zeros ( 1, 2 );
  if ( x(1) <= 0.0 )
    t2   = (-x(1)) ^ tau;
    g(1) = theta * phi * t2 * tau / x(1);
    g(2) = 1 + (2 * x(2));
  else
   t1   = x(1) ^ tau;
   g(1) = theta * t1 * tau / x(1);
   g(2) = 1 + (2 * x(2));
  end
end

%*****************************************************************************80
function h = eval_h ( x )
  tau   = 2.0;
  theta = 6.0;
  phi   = 60.0;

  h = zeros ( 2, 2 );
  if ( x(1) <= 0.0 )
    t1 = theta * phi;
    t2 = (-x(1)) ^ tau;
    t3 = tau ^ 2;
    t5 = x(1) ^ 2;
    t6 = 1 / t5;
    h(1,1) = t1 * t2 * t3 * t6 - t1 * t2 * tau * t6;
    h(1,2) = 0;
    h(2,1) = 0;
    h(2,2) = 2;
  else
    t1 = x(1) ^ tau;
    t2 = theta * t1;
    t3 = tau ^ 2;
    t4 = x(1) ^ 2;
    t5 = 1 / t4;
    h(1,1) = t2 * t3 * t5 - t2 * tau * t5;
    h(1,2) = 0;
    h(2,1) = 0;
    h(2,2) = 2;
  end
end
