%
% Trust region minimization with exact solution of trusted step.
%
%  USAGE:
%
%  x = LevenbergMarquardt( fun, Grad, Hess, x0, epsi )
%
%  or
%
% [x,g,radius] = LevenbergMarquardt( fun, Grad, Hess, x0, epsi )
%
function [X,FF,NG] = LevenbergMarquardt( fun, x0, varargin )  

  opts = { 'tol', 'maxiter', 'beta1', 'beta2' } ;
  defs = {  1E-8,       100,    0.25,    0.75 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  beta1   = OPTS.beta1 ;
  beta2   = OPTS.beta2 ;
  maxiter = OPTS.maxiter ;
  tol     = OPTS.tol ;

  x  = x0 ;

  Id = eye(length(x));

  X  = [] ;
  FF = [] ;
  NG = [] ;

  % compute model
  F  = feval( fun, 'F',  x ) ;
  JF = feval( fun, 'JF', x ) ;
  H  = JF'*JF ;
  f  = (1/2) * norm(F,2)^2 ;
  mu = norm( JF, 'fro' ) ;

  for k=1:maxiter

    % gradient direction
    g  = JF'*F ;
    
    % save values
    X  = [ X x ] ;
    FF = [ FF f ] ;
    NG = [ NG norm(g,'inf') ] ;

    done = false ;
    while ~done

      % compute the step
      s = - (H+mu*Id) \ g ;

      % check if it is a descent direction
      alpha = dot(s,g) ;
      if alpha < 0
        % perform step
        xnew  = x + s ;
        Fnew  = feval( fun, 'F',  xnew ) ;
        JFnew = feval( fun, 'JF',  xnew ) ;
        gnew  = JFnew' * Fnew ;
        fnew  = (1/2) * norm(Fnew,2)^2 ;

        if norm(s,inf) < tol & norm(gnew,inf) < tol
          X  = [ X  xnew ] ;
          FF = [ FF fnew ] ;
          NG = [ NG norm(gnew,'inf') ] ;
          disp('solution found!');
          return ;
        end;

        if mu > 1E100
          disp('lambda too big!');
          return ;
        end;
        
        % check the reduction
        ared = f - fnew ;
        pred = - alpha - 0.5 * (dot(s,H*s)+mu*dot(s,s)) ;
        rho  = ared / pred ;
      
        if rho >= 0
          done = true ;
          x    = xnew ;

          % compute model
          F  = feval( fun, 'F',  x ) ;
          JF = feval( fun, 'JF', x ) ;
          H  = JF'*JF ;
          f  = (1/2) * norm(F,2)^2 ;

          % enlarge or reduce trust region depending on ratio rho.
          if rho < beta1
            % too few reduction, reduce trust region
            mu = 2 * mu ;
          elseif rho > beta2
            % very good reduction, enlarge trust region
            mu = mu/2 ;
          end
        else
          % no reduction, reduce trust region
          mu = 2 * mu ;
          fprintf(1,'reject (no reduction) iter=%d mu=%g rho=%g\n',k,mu,rho) ;
        end

      else
        % if not a descent direction reduce trust region
        mu = 2 * mu ;          
        fprintf(1,'reject (not descent) iter=%d mu=%g\n',k,mu) ;
      end
    end % end while
  end ;
end
