%
% Trust region minimization with exact solution of trusted step.
%
%  USAGE:
%
%  x = TrustRegion( fun, x0, epsi )
%
%  or
%
% [x,g,radius] = TrustRegion( fun, x0, epsi )
%
function [ierr,X,F,NG] = TrustRegion( fun, x0, varargin )
  ierr = 0 ;
  opts = { 'tol', 'maxiter', 'maxsubiter', 'beta1', 'beta2' };
  defs = {  1E-8,       100,           20,     0.1,    0.99 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  beta1      = OPTS.beta1 ;
  beta2      = OPTS.beta2 ;
  maxiter    = OPTS.maxiter ;
  maxsubiter = OPTS.maxsubiter ;
  tol        = OPTS.tol ;

  x = x0 ;
  f = feval( fun, 'f',   x ) ;
  g = feval( fun, 'Df',  x ) ;
  H = feval( fun, 'DDf', x ) ;

  radius = norm(g) ;

  X  = [] ;
  F  = [] ;
  NG = [] ;

  for k=1:maxiter

    % compute model
    g = feval( fun, 'Df',  x ) ;
    H = feval( fun, 'DDf', x ) ;

    % save values
    X  = [ X x ] ;
    F  = [ F f ] ;
    NG = [ NG norm(g,'inf') ] ;

    done    = false ;
    subiter = 0 ;
    while ~done
      % compute the step
      s    = TrustRegionStep( radius, g', H ) ;
      if ~isfinite(s)
        ierr = 3 ;
        return ;
      end
      
      xnew = x + s ;
      fnew = feval( fun, 'f', xnew ) ;

      if norm(s,inf) < tol & norm(g,inf) < tol
        disp('solution found!');
        return ;
      end;
    
      % check the reduction
      ared = f - fnew ; 
      pred = - dot( g, s )  - 0.5 * dot(s,H*s) ;
      rho  = ared / pred ;
      
      if rho >= beta2 % very successfull augment radius
        done   = true ;
        x      = xnew ;
        f      = fnew ;
        radius = max(radius, 2*norm(s)) ;
      elseif rho >= beta1 % successful 
        done   = true ;
        x      = xnew ;
        f      = fnew ;
      else
        fprintf(1, 'reject iter=%d radius=%g\n',k,radius) ;
        radius = radius / 2 ;
      end
      subiter = subiter + 1 ;
      if ~done & subiter >= maxsubiter ;
        ierr = 2 ;
        return ;
      end
    end
  end ;
  ierr = 1 ;
end

%
%  Trust region minimization with exact solution of trusted step.
%
%  USAGE:
%
%  x = TrustRegionStep( radius, g, H )
%
function s = TrustRegionStep( radius, g, H )
  mu = radius ;
  ID = eye(length(g)) ;
  for i=1:50
    [Q,R] = qr(H+mu*ID) ;
    D     = abs(diag(R,0)) ;
    if ~isfinite(mu) | ~isfinite(D) | max(D) > 1E10*min(D)
      rcond(H+mu*ID)
      s = NaN ;
      fprintf(1, 'TrustRegionStep not converged, mu = %g\n', mu) ;
      return ;
    end
    % compute the model
    s    = -R\(Q'*g) ;
    s1   = -R\(Q'*s) ;
    ss   = sqrt(dot(s,s)) ;
    phi  = ss - radius ;
    phi1 = dot(s,s1) / ss ;
    % update mu and s
    mu   = mu - (ss/radius) * (phi/phi1) ;
    %mu   = mu - (phi/phi1) ;
    if mu < 0 
      s  = -(H \ g) ; % point internal to trust region
      return ;
    end
    if abs(phi) < radius*1E-6 
      return ;
    end;
  end
  fprintf(1, 'TrustRegionStep not converged, phi = %g\n', phi) ;
end

