function [ierr,X,Flist] = dfsane( par, fn, varargin )

  opts = { 'tol', 'maxiter', 'M', 'method', 'noImprovement' } ;
  defs = {  1E-8,      1000,  50,        2,             100 } ;

  OPTS = parseArgs( opts, defs, varargin{:} ) ;

  tol           = OPTS.tol ;
  maxiter       = OPTS.maxiter ;
  M             = OPTS.M ;
  method        = OPTS.method ;
  noImprovement = OPTS.noImprovement ;

  n      = length(par) ;
  fcnt   = 0 ;
  iter   = 0 ;
  bl     = 0 ;
  alfa   = 1 ;
  eta    = 1 ;
  eps    = 1E-10 ;
  lastfv = zeros(1,M) ;

  F     = feval(fn,'F',par) ;
  F0    = sqrt(dot(F,F)) ;
  normF = F0 ;

  X     = [par] ;
  Flist = [normF] ;

  pbest          = par ;
  normF_best     = normF ;
  lastfv(1)      = normF^2 ;
  flag           = 0 ;
  knoImprovement = 0 ;

  while normF/sqrt(n) > tol & iter <= maxiter

    if ( (abs(alfa) <= eps) | (abs(alfa) >= 1/eps) )
      if normF > 1 
        alpha = 1 ;
      elseif normF >= 1e-05 & normF <= 1
        alpha = 1/normF ;
      elseif normF < 1e-05 
        alpha = 1e+05 ;
      end
    end

    % Steplength for first iteration is scaled properly
    if iter == 0
      alfa  = min(1/normF, 1) ;
      alfa1 = alfa ;
      alfa2 = alfa ;
    end

    temp  = alfa2 ;
    alfa2 = alfa ;
    if normF <= 0.01
      alfa = alfa1 ; % retard scheme
    end
    alfa1 = temp ;

    [ flag, blout, pnew, Fnew, fune ] = lsm( par, fn, F, dot(F,F), alfa, lastfv, eta, bl ) ;
    bl = blout ;

    if flag > 0 
      break ;
    end

    pF = dot( pnew - par, Fnew - F   ) ;
    pp = dot( pnew - par, pnew - par ) ;
    FF = dot( Fnew - F, Fnew - F ) ;

    switch method
    case 1; alfa = pp/pF ;
    case 2; alfa = pF/FF ;
    case 3; alfa = sign(pF) * sqrt(pp/FF) ;
    end

    if ~isfinite(alfa) 
      alfa = eps ;
    end

    par   = pnew ;
    F     = Fnew ;
    fun   = fune ;
    normF = sqrt(fun) ;

    X     = [X     par   ] ;
    Flist = [Flist normF ] ;

    if normF < normF_best
      pbest      = par ;
      normF_best = normF ;
      knoImprovement     = 0 ;
    else
      knoImprovement = knoImprovement + 1 ;
    end

    iter = iter + 1 ;
    lastfv( 1 + mod(iter,M)) = fun ;

    eta = F0/(iter + 1)^2 ;

    % Iterations are stopped when there is no decrease in ||F(x_n|| over `noImprovement' consecutive iterations
    if knoImprovement == noImprovement
      flag = 3 ;
      break ;
    end
  end % main loop complete

  ierr = 1 ;
  if flag == 0
    if normF_best/sqrt(n) <= tol
      ierr = 0 ;
      fprintf( 1, 'Successful convergence\n' ) ;
    else
      fprintf( 1, 'Maximum limit for iterations exceeded\n' ) ;
    end
    return ;
  elseif flag == 1
    fprintf( 1, 'Failure: Error in function evaluation\n' ) ;
  elseif flag == 2
    fprintf( 1, 'Failure: Maximum limit on steplength reductions exceeded\n' ) ;
  elseif flag == 3 
    fprintf( 1, 'Lack of improvement in objective function\n' ) ;
  end
end

function [ lsflag, bl, xnew, Fnew, fune ] = lsm( x, fn, F, fval, alfa, lastfv, eta, blin )
  maxbl  = 100 ;
  gamma  = 1E-04 ;
  sigma1 = 0.1 ;
  sigma2 = 0.5 ;
  lam1   = 1 ;
  lam2   = 1 ;
  cbl    = 0 ;
  fmax   = max(lastfv) ;

  bl     = blin ;
  lsflag = 0 ;
  xnew   = NaN ;
  Fnew   = NaN ;
  fune   = NaN ;

  while cbl < maxbl
    d    = -alfa * F ;
    xnew = x + lam1 * d ;
    Fnew = feval( fn, 'F', xnew ) ;
    if ~all(isfinite(Fnew))
      lsflag = 1   ;
      return ;
    end

    fune1 = dot(Fnew,Fnew) ;
    if fune1 <= fmax + eta - (lam1^2 * gamma * fval)
      if cbl >= 1 
        bl = bl + 1 ;
      end
      lsflag = 0 ;
      fune   = fune1 ;
      return ;
    end

    xnew = x - lam2 * d ;
    Fnew = feval( fn, 'F', xnew ) ;
    if ~all(isfinite(Fnew))
      lsflag = 1 ;
    end

    fune2 = dot(Fnew,Fnew) ;
    if fune2 <= fmax + eta - (lam2^2 * gamma * fval)
      if cbl >= 1
        bl     = bl + 1 ;
        lsflag = 0 ;
        fune   = fune2 ;
        return ;
      end
    end

    lamc = lam1^2/(fune1/fval + (2 * lam1 - 1) ) ;
    lam1 = max( sigma1 * lam1, min(sigma2 * lam1, lamc ) ) ;
 
    lamc = lam2^2/(fune2/fval + (2 * lam2 - 1) ) ;
    lam2 = max( sigma1 * lam2, min(sigma2 * lam2, lamc ) ) ;

    cbl = cbl + 1 ;
  end
  lsflag = 2 ;
  fune   = fune ;
end
